from typing import List

from .adapter import Adapter
from .enums import Formality
from .enums import PreserveFormatting as PF
from .enums import SourceLang as SL
from .enums import SplitSentences as SS
from .enums import TargetLang as TL

__all__ = ['Translator']


class Translator:
    def __init__(self, adapter: Adapter) -> None:
        self._adapter = adapter

    def translate(self, text, *, target_lang: TL,
                  source_lang: SL = None, split_sentences: SS = None,
                  preserve_formatting: PF = None, formality: Formality = None) -> str:
        payload = {
            'text': text,
            'target_lang': target_lang.value
        }
        if source_lang:
            payload['source_lang'] = source_lang.value
        if split_sentences:
            payload['split_sentences'] = split_sentences.value
        if preserve_formatting:
            payload['preserve_formatting'] = preserve_formatting.value
        if formality:
            payload['formality'] = formality.value
        return self._adapter.get_translated_text(payload)

    def translate_multi(self, text_list: list, *, target_lang: TL,
                        source_lang: SL = None, split_sentences: SS = None,
                        preserve_formatting: PF = None, formality: Formality = None) -> List[str]:
        payload = {
            'text': text_list,
            'target_lang': target_lang.value
        }
        if source_lang:
            payload['source_lang'] = source_lang.value
        if split_sentences:
            payload['split_sentences'] = split_sentences.value
        if preserve_formatting:
            payload['preserve_formatting'] = preserve_formatting.value
        if formality:
            payload['formality'] = formality.value
        return self._adapter.get_translated_text_multi(payload)

    def usage(self) -> dict:
        return self._adapter.get_usage()

    def supported_languages(self) -> List[dict]:
        return self._adapter.get_supported_languages()



from enum import Enum

__all__ = [
    'SourceLang',
    'TargetLang',
    'SplitSentences',
    'PreserveFormatting',
    'Formality'
]


class SourceLang(Enum):
    Bulgarian = 'BG'
    Czech = 'CS'
    Danish = 'DA'
    German = 'DE'
    Greek = 'EL'
    English = 'EN'
    Spanish = 'ES'
    Estonian = 'ET'
    Finnish = 'FI'
    French = 'FR'
    Hungarian = "HU"
    Italian = "IT"
    Japanese = "JA"
    Lithuanian = "LT"
    Latvian = "LV"
    Dutch = "NL"
    Polish = "PL"
    Portuguese = "PT"
    Romanian = "RO"
    Russian = "RU"
    Slovak = "SK"
    Slovenian = "SL"
    Swedish = "SV"
    Chinese = "ZH"


class TargetLang(Enum):
    Bulgarian = 'BG'
    Czech = 'CS'
    Danish = 'DA'
    German = 'DE'
    Greek = 'EL'
    English = 'EN'
    English_GB = 'EN-GB'
    English_US = 'EN-US'
    Spanish = 'ES'
    Estonian = 'ET'
    Finnish = 'FI'
    French = 'FR'
    Hungarian = 'HU'
    Italian = 'IT'
    Japanese = 'JA'
    Lithuanian = 'LT'
    Latvian = 'LV'
    Dutch = 'NL'
    Polish = 'PL'
    Portuguese = 'PT-PT'
    Portuguese_BR = 'PT-BR'
    Portuguese_PT = 'PT'
    Romanian = 'RO'
    Russian = 'RU'
    Slovak = 'SK'
    Slovenian = 'SL'
    Swedish = 'SV'
    Chinese = 'ZH'


class SplitSentences(Enum):
    enabled = 0
    disabled = 1


class PreserveFormatting(Enum):
    respect = 1
    ignore = 2


class Formality(Enum):
    default = 'default'
    more = 'more'
    less = 'less'



import json
from abc import ABCMeta, abstractmethod
from typing import List, Optional, Union

import aiohttp
import requests

from .errors import (BadRequest, Forbidden, HTTPException, InternalServerError,
                     NotFound, PayloadTooLarge, QuotaExceeded,
                     ServiceUnavailable, TooManyRequests, URITooLong)

__all__ = ['Adapter', 'RequestsAdapter', 'AiohttpAdapter']


class Adapter(metaclass=ABCMeta):
    def __init__(self, authentication_key: str, *, pro: bool = False) -> None:
        self.base_url_free = 'https://api-free.deepl.com/'
        self.base_url_pro = 'https://api.deepl.com/'
        self.api_version = 'v2'
        self.auth_key = authentication_key
        self.pro = pro

    @abstractmethod
    def request(self, method: str,
                path: str, payload: dict = {}, **kwargs) -> Optional[Union[list, dict]]:
        raise NotImplementedError()

    @abstractmethod
    def get_translated_text(self, payload: dict) -> str:
        raise NotImplementedError()

    @abstractmethod
    def get_translated_text_multi(self, payload: dict) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def get_usage(self) -> dict:
        raise NotImplementedError()

    @abstractmethod
    def get_supported_languages(self) -> List[dict]:
        raise NotImplementedError()

    def _check_status(self, status_code, response, data) -> Union[dict, list]:
        if 200 <= status_code < 300:
            return data
        message = data.get('message', '') if data else ''
        if status_code == 400:
            raise BadRequest(response, message)
        elif status_code == 403:
            raise Forbidden(response, message)
        elif status_code == 404:
            raise NotFound(response, message)
        elif status_code == 413:
            raise PayloadTooLarge(response, message)
        elif status_code == 414:
            raise URITooLong(response, message)
        elif status_code == 429:
            raise TooManyRequests(response, message)
        elif status_code == 456:
            raise QuotaExceeded(response, message)
        elif status_code == 503:
            raise ServiceUnavailable(response, message)
        elif 500 <= status_code < 600:
            raise InternalServerError(response, message)
        else:
            raise HTTPException(response, message)


class RequestsAdapter(Adapter):
    def request(self, method: str,
                path: str, payload: dict = {}, **kwargs) -> Optional[Union[list, dict]]:
        payload['auth_key'] = self.auth_key

        url = self.base_url_pro if self.pro else self.base_url_free
        url += self.api_version + path

        resp = requests.request(method, url, data=payload, **kwargs)
        try:
            data = resp.json()
        except json.JSONDecodeError:
            data = None
        return self._check_status(resp.status_code, resp, data)

    def get_translated_text(self, payload) -> str:
        data = self.request('POST', '/translate', payload)
        return data['translations'][0]['text']

    def get_translated_text_multi(self, payload: dict) -> List[str]:
        data = self.request('POST', '/translate', payload)
        return [s['text'] for s in data['translations']]

    def get_usage(self) -> dict:
        data = self.request('POST', '/usage')
        return data

    def get_supported_languages(self) -> List[dict]:
        data = self.request('POST', '/languages')
        return data


class AiohttpAdapter(Adapter):

    async def request(self, method: str,
                      path: str, payload: dict = {}, **kwargs) -> Optional[Union[list, dict]]:
        payload['auth_key'] = self.auth_key

        url = self.base_url_pro if self.pro else self.base_url_free
        url += self.api_version + path

        async with aiohttp.request(
                method, url, data=payload, **kwargs) as resp:
            try:
                data = await resp.json(content_type=None)
            except json.JSONDecodeError:
                data = None
            status_code = resp.status
        return self._check_status(status_code, resp, data)

    async def get_translated_text(self, payload: dict) -> str:
        data = await self.request('POST', '/translate', payload)
        return data['translations'][0]['text']

    async def get_translated_text_multi(self, payload: dict) -> List[str]:
        data = await self.request('POST', '/translate', payload)
        return [s['text'] for s in data['translations']]

    async def get_usage(self) -> dict:
        data = await self.request('POST', '/usage')
        return data

    async def get_supported_languages(self) -> List[dict]:
        data = await self.request('POST', '/languages')
        return data



import aiohttp
import requests

__all__ = [
    'DeepLException',
    'HTTPException',
    'BadRequest',
    'Forbidden',
    'NotFound',
    'PayloadTooLarge',
    'URITooLong',
    'TooManyRequests',
    'QuotaExceeded',
    'ServiceUnavailable',
    'InternalServerError'
]


class DeepLException(Exception):
    pass


class HTTPException(DeepLException):
    """Exception raised when an HTTP request fails.
    Attributes
    ------------
    response: Union[:class:`aiohttp.ClientResponse`, :class:`requests.Response`]
        If you are using the RequestsAdapter,
        :class:`requests.Response`.
        If you are using the AiohttpAdapter,
        :class:`aiohttp.ClientResponse`.
    message: :class:`str`
        The Messages returned by DeepL API.
    status: :class:`int`
        The status code of the HTTP request.
    """

    def __init__(self, response, messega) -> None:
        self.response = response
        self.message = messega or 'No error message was sent from the DeepL API.'
        if isinstance(response, aiohttp.ClientResponse):
            self.status = response.status
        elif isinstance(response, requests.Response):
            self.status = response.status_code

        super().__init__(f'{self.status} {response.reason}: {self.message}')


class BadRequest(HTTPException):
    """Bad request. Please check error message and your parameters.

    Subclass of :exc:`HTTPException`
    """
    pass


class Forbidden(HTTPException):
    """Authorization failed. Please supply a valid auth_key parameter.

    Subclass of :exc:`HTTPException`
    """
    pass


class NotFound(HTTPException):
    """The requested resource could not be found.

    Subclass of :exc:`HTTPException`
    """
    pass


class PayloadTooLarge(HTTPException):
    """The request size exceeds the limit.

    Subclass of :exc:`HTTPException`
    """
    pass


class URITooLong(HTTPException):
    """The request URL is too long.
    You can avoid this error by using a POST request instead of a GET request.

    Subclass of :exc:`HTTPException`
    """
    pass


class TooManyRequests(HTTPException):
    """Too many requests. Please wait and resend your request.

    Subclass of :exc:`HTTPException`
    """
    pass


class QuotaExceeded(HTTPException):
    """Quota exceeded. The character limit has been reached.

    Subclass of :exc:`HTTPException`
    """
    pass


class ServiceUnavailable(HTTPException):
    """Resource currently unavailable. Try again later.

    Subclass of :exc:`HTTPException`
    """
    pass


class InternalServerError(HTTPException):
    """	Internal error

    Subclass of :exc:`HTTPException`
    """
    pass



"""
DeepL API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the DeepL API.
:copyright: (c) 2021-present grarich
:license: MIT, see LICENSE for more details.
"""

__title__ = 'deepl'
__author__ = 'grarich'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021-present grarich'
__version__ = '1.0.1'

__path__ = __import__('pkgutil').extend_path(__path__, __name__)

from .adapter import Adapter, AiohttpAdapter, RequestsAdapter
from .enums import Formality, PreserveFormatting, SourceLang, SplitSentences, TargetLang
from .translator import Translator

__all__ = [
    'Adapter',
    'RequestsAdapter',
    'AiohttpAdapter',
    'SourceLang',
    'TargetLang',
    'SplitSentences',
    'PreserveFormatting',
    'Formality',
    'Translator'
]



import re

import setuptools

version = ''
with open('deepl/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

requirements = []
with open('requirements.txt', 'r') as f:
    requirements = f.read().splitlines()

with open('README.md', 'r') as f:
    long_description = f.read()

setuptools.setup(
    name='deepl.py',
    version=version,
    author='grarich',
    author_email='grarich123@gmail.com',
    description='A Python wrapper for the DeepL API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/grarich123/deepl.py',
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
